package com.sun.jndi.ldap;

import java.util.Arrays;
import java.util.Locale;
import java.lang.reflect.Method;
import javax.net.SocketFactory;
import java.io.OutputStream;
import javax.naming.ldap.Control;

class ClientId
{
    private final int version;
    private final String hostname;
    private final int port;
    private final String protocol;
    private final Control[] bindCtls;
    private final OutputStream trace;
    private final String socketFactory;
    private final int myHash;
    private final int ctlHash;
    private SocketFactory factory;
    private Method sockComparator;
    private boolean isDefaultSockFactory;
    public static final boolean debug = false;
    
    ClientId(int version, String s, int port, String protocol, Control[] array, OutputStream trace, String socketFactory) {
        this.factory = null;
        this.sockComparator = null;
        this.isDefaultSockFactory = false;
        this.version = version;
        this.hostname = s.toLowerCase(Locale.ENGLISH);
        this.port = port;
        this.protocol = protocol;
        this.bindCtls = ((array != null) ? ((Control[])array.clone()) : null);
        this.trace = trace;
        this.socketFactory = socketFactory;
        if (socketFactory != null && !socketFactory.equals("javax.net.ssl.SSLSocketFactory")) {
            try {
                Class<Object> loadClass = Obj.helper.loadClass(socketFactory);
                Class<Object> forName = Class.forName("java.lang.Object");
                this.sockComparator = loadClass.getMethod("compare", new Class[] { forName, forName });
                this.factory = (SocketFactory)loadClass.getMethod("getDefault", new Class[0]).invoke(null, new Object[0]);
            }
            catch (Exception ex) {}
        }
        else {
            this.isDefaultSockFactory = true;
        }
        int n = version + port + ((trace != null) ? trace.hashCode() : 0) + ((this.hostname != null) ? this.hostname.hashCode() : 0) + ((protocol != null) ? protocol.hashCode() : 0);
        int hashCodeControls = hashCodeControls(array);
        this.ctlHash = hashCodeControls;
        this.myHash = n + hashCodeControls;
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof ClientId)) {
            return false;
        }
        ClientId clientId = (ClientId)o;
        return this.myHash == clientId.myHash && this.version == clientId.version && this.port == clientId.port && this.trace == clientId.trace && (this.hostname == clientId.hostname || (this.hostname != null && this.hostname.equals(clientId.hostname))) && (this.protocol == clientId.protocol || (this.protocol != null && this.protocol.equals(clientId.protocol))) && this.ctlHash == clientId.ctlHash && equalsControls(this.bindCtls, clientId.bindCtls) && this.equalsSockFactory(clientId);
    }
    
    @Override
    public int hashCode() {
        return this.myHash;
    }
    
    private static int hashCodeControls(Control[] array) {
        if (array == null) {
            return 0;
        }
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            n = n * 31 + array[i].getID().hashCode();
        }
        return n;
    }
    
    private static boolean equalsControls(Control[] array, Control[] array2) {
        if (array == array2) {
            return true;
        }
        if (array == null || array2 == null) {
            return false;
        }
        if (array.length != array2.length) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if (!array[i].getID().equals(array2[i].getID()) || array[i].isCritical() != array2[i].isCritical() || !Arrays.equals(array[i].getEncodedValue(), array2[i].getEncodedValue())) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private boolean equalsSockFactory(ClientId clientId) {
        if (this.isDefaultSockFactory && clientId.isDefaultSockFactory) {
            return true;
        }
        if (!clientId.isDefaultSockFactory) {
            return this.invokeComparator(clientId, this);
        }
        return this.invokeComparator(this, clientId);
    }
    
    private boolean invokeComparator(ClientId clientId, ClientId clientId2) {
        Object invoke;
        try {
            invoke = clientId.sockComparator.invoke(clientId.factory, new Object[] { clientId.socketFactory, clientId2.socketFactory });
        }
        catch (Exception ex) {
            return false;
        }
        return (int)invoke == 0;
    }
    
    private static String toStringControls(Control[] array) {
        if (array == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < array.length; ++i) {
            sb.append(array[i].getID());
            sb.append(' ');
        }
        return sb.toString();
    }
    
    @Override
    public String toString() {
        return this.hostname + ":" + this.port + ":" + ((this.protocol != null) ? this.protocol : "") + ":" + toStringControls(this.bindCtls) + ":" + this.socketFactory;
    }
}
